unit fMain;

interface

uses
  Forms, StdCtrls, Classes, Controls, ExtCtrls;

type

  TTimeChangeEvent = procedure(Sender: TObject; DateTime: TDateTime) of object;
  TTextChangeEvent = procedure(Sender: TObject; Index: Integer; Text: string) of object;

{ TfrmMain }
  TfrmMain = class(TForm)
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Edit5: TEdit;
    Label1: TLabel;
    Timer1: TTimer;
    procedure Timer1Timer(Sender: TObject);
  private
    FOnTimeChange: TTimeChangeEvent;
    FOnTextChange: TTextChangeEvent;
    function GetEdits(Index: Integer): TEdit;
    function GetTexts(Index: Integer): string;
    procedure SetTexts(Index: Integer; const Value: string);
    function GetConnectionCount: Integer;
  protected
    FConnections: TInterfaceList;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Edits[Index: Integer]: TEdit read GetEdits;
    property Texts[Index: Integer]: string read GetTexts write SetTexts;
    property Connections: TInterfaceList read FConnections;
    property ConnectionCount: Integer read GetConnectionCount;
    property OnTimeChange: TTimeChangeEvent read FOnTimeChange write FOnTimeChange;
    property OnTextChange: TTextChangeEvent read FOnTextChange write FOnTextChange;
  end;

var
  frmMain: TfrmMain;

implementation

uses SysUtils;

{$R *.dfm}

{ TfrmMain }

constructor TfrmMain.Create(AOwner: TComponent);
begin
  inherited;
  FConnections := TInterfaceList.Create;
  Randomize;
end;

destructor TfrmMain.Destroy;
begin
  FreeAndNil(FConnections);
  inherited;
end;

function TfrmMain.GetConnectionCount: Integer;
begin
  Result := FConnections.Count;
end;

function TfrmMain.GetEdits(Index: Integer): TEdit;
begin
  Result := FindComponent('Edit'+IntToStr(Index)) as TEdit;
end;

function TfrmMain.GetTexts(Index: Integer): string;
begin
  if Assigned(Edits[Index]) then Result := Edits[Index].Text else Result := '';
end;

procedure TfrmMain.SetTexts(Index: Integer; const Value: string);
begin
  if Assigned(Edits[Index]) then
    if Value <> Edits[Index].Text then
    begin
      Edits[Index].Text := Value;
      if Assigned(FOnTextChange) then
        FOnTextChange(Self, Index, Value);
    end;
end;

procedure TfrmMain.Timer1Timer(Sender: TObject);
begin
  Label1.Caption := FormatDateTime('hh:nn:ss.zzz', Now) + '  ' +
                    Format('su=%d', [GetConnectionCount]);
  if Assigned(FOnTimeChange) then FOnTimeChange(Self, Now);
end;

end.
