unit fTestFuncIMPL;

{$WARN SYMBOL_PLATFORM OFF}

interface

uses
  ComObj, ActiveX, AxCtrls, ServerA_TLB, StdVcl;
  
type

{ TTestFunc }
  TTestFunc = class(TAutoObject, IConnectionPointContainer, ITestFunc)
  private
    FEvents: IUnknown;
    FConnectionPoints: TConnectionPoints;
    FConnectionPoint: TConnectionPoint;
    procedure FireTimeChangeEvent(Sender: TObject; DateTime: TDateTime);
    procedure FireTextChangeEvent(Sender: TObject; Index: Integer; Text: string);
  public
    procedure Initialize; override;
  protected
    property ConnectionPoints: TConnectionPoints read FConnectionPoints implements IConnectionPointContainer;
    procedure EventSinkChanged(const EventSink: IUnknown); override;
    function GetConnectionCount: Integer; safecall;
    function GetTexts(Index: Integer): WideString; safecall;
    procedure SetTexts(Index: Integer; const Value: WideString); safecall;
  end;

implementation

uses ComServ, fMain;

procedure TTestFunc.EventSinkChanged(const EventSink: IUnknown);
var
	Index: Integer;
begin
  with frmMain.Connections do
  begin
    Lock;
    try
      if EventSink <> nil then
      begin
        FEvents := EventSink;
        Add(FEvents);
      end
      else if FEvents <> nil then
      begin
        Index := IndexOf(FEvents);
        if Index <> -1 then
          Delete(Index);
        FEvents := nil;
      end;
    finally
      Unlock;
    end;
  end;
end;

procedure TTestFunc.Initialize;
begin
  inherited Initialize;
  FConnectionPoints := TConnectionPoints.Create(Self);
  if AutoFactory.EventTypeInfo <> nil then
    FConnectionPoint := FConnectionPoints.CreateConnectionPoint(AutoFactory.EventIID, ckSingle, EventConnect)
  else FConnectionPoint := nil;
  frmMain.OnTimeChange := FireTimeChangeEvent;
  frmMain.OnTextChange := FireTextChangeEvent;
end;

function TTestFunc.GetConnectionCount: Integer;
begin
  Result := frmMain.ConnectionCount;
end;

function TTestFunc.GetTexts(Index: Integer): WideString;
begin
  Result := frmMain.Texts[Index];
end;

procedure TTestFunc.SetTexts(Index: Integer; const Value: WideString);
begin
  frmMain.Texts[Index] := Value;
end;

procedure TTestFunc.FireTimeChangeEvent(Sender: TObject; DateTime: TDateTime);
var
	P: IUnknown;
	I, C: Integer;
begin
  with frmMain, Connections do
  begin
	  C := ConnectionCount;
    if C > 0 then
    begin
      Lock;
      try
        for I := C-1 downto 0 do
        begin
          P := Connections[I];
          if P <> nil then
            (P as ITestFuncEvents).OnTimeChange(DateTime);
        end;
      finally
        Unlock;
      end;
    end;
  end;
end;

procedure TTestFunc.FireTextChangeEvent(Sender: TObject; Index: Integer; Text: string);
var
	P: IUnknown;
	I, C: Integer;
begin
  with frmMain, Connections do
  begin
	  C := ConnectionCount;
    if C > 0 then
    begin
      Lock;
      try
        for I := C-1 downto 0 do
        begin
          P := Connections[I];
          if P <> nil then
            (P as ITestFuncEvents).OnTextChange(Index, Text);
        end;
      finally
        Unlock;
      end;
    end;
  end;
end;

initialization
  TAutoObjectFactory.Create(ComServer, TTestFunc, Class_TestFunc, ciMultiInstance, tmApartment);
end.
